@@ -1,6 +1,6 @@
 /*****
 *
-* Copyright (C) 2003-2017 CS-SI. All Rights Reserved.
+* Copyright (C) 2003-2019 CS-SI. All Rights Reserved.
 * Author: Nicolas Delon <nicolas.delon@prelude-ids.com>
 *
 * This file is part of the PreludeDB library.
@@ -233,10 +233,33 @@ static int sql_build_limit_offset_string(void *session, int limit, int offset, p
 
 
 
+static int sql_query_prepare(preludedb_sql_t *sql, preludedb_sql_query_t *query, prelude_string_t *output)
+{
+        int ret;
+        int64_t limit = -1, offset = -1;
+
+        ret = prelude_string_cat(output, preludedb_sql_query_get_string(query));
+        if ( ret < 0 )
+                return ret;
+
+        if ( preludedb_sql_query_get_option(query, PRELUDEDB_SQL_QUERY_OPTION_FOR_UPDATE, NULL) ) {
+                ret = prelude_string_cat(output, " FOR UPDATE");
+                if ( ret < 0 )
+                        return ret;
+        }
+
+        preludedb_sql_query_get_option(query, PRELUDEDB_SQL_QUERY_OPTION_LIMIT, &limit);
+        preludedb_sql_query_get_option(query, PRELUDEDB_SQL_QUERY_OPTION_OFFSET, &offset);
+
+        return preludedb_sql_build_limit_offset_string(sql, limit, offset, output);
+}
+
+
 
 static int sql_query(void *session, const char *query, preludedb_sql_table_t **table)
 {
         int ret;
+        unsigned long long afrows;
         MYSQL_RES *result;
 
         ret = mysql_query(session, query);
@@ -245,6 +268,7 @@ static int sql_query(void *session, const char *query, preludedb_sql_table_t **t
 
         do {
                 result = mysql_store_result(session);
+                afrows = mysql_affected_rows(session);
                 if ( ! result && mysql_field_count(session) > 0 )
                         return handle_error(session, PRELUDEDB_ERROR_QUERY);
 
@@ -266,9 +290,9 @@ static int sql_query(void *session, const char *query, preludedb_sql_table_t **t
          * Last result
          */
         if ( ! result )
-                return 0;
+                return (int) afrows;
 
-        if ( mysql_num_rows(result) == 0 ) {
+        if ( afrows == 0 ) {
                 mysql_free_result(result);
                 return 0;
         }
@@ -279,7 +303,7 @@ static int sql_query(void *session, const char *query, preludedb_sql_table_t **t
                 return ret;
         }
 
-        return 1;
+        return (int) afrows;
 }
 
 
@@ -633,6 +657,7 @@ int mysql_LTX_preludedb_plugin_init(prelude_plugin_entry_t *pe, void *data)
         preludedb_plugin_sql_set_open_func(plugin, sql_open);
         preludedb_plugin_sql_set_close_func(plugin, sql_close);
         preludedb_plugin_sql_set_escape_binary_func(plugin, sql_escape_binary);
+        preludedb_plugin_sql_set_query_prepare_func(plugin, sql_query_prepare);
         preludedb_plugin_sql_set_query_func(plugin, sql_query);
         preludedb_plugin_sql_set_get_server_version_func(plugin, sql_get_server_version);
         preludedb_plugin_sql_set_table_destroy_func(plugin, sql_table_destroy);
