@@ -1,6 +1,6 @@
 /*****
 *
-* Copyright (C) 2003-2017 CS-SI. All Rights Reserved.
+* Copyright (C) 2003-2019 CS-SI. All Rights Reserved.
 * Author: Nicolas Delon <nicolas.delon@prelude-ids.com>
 *
 * This file is part of the PreludeDB library.
@@ -82,7 +82,8 @@ static int handle_error(prelude_error_code_t code, PGconn *conn)
 
 static int _sql_query(void *session, const char *query, PGresult **result)
 {
-        int status, ntuple;
+        char *tmp;
+        int status, ntuple = 0;
 
         *result = PQexec(session, query);
         if ( ! *result )
@@ -97,35 +98,63 @@ static int _sql_query(void *session, const char *query, PGresult **result)
                 return ntuple;
         }
 
+        tmp = PQcmdTuples(*result);
+        if ( tmp )
+                ntuple = atoi(tmp);
+
         PQclear(*result);
+        *result = NULL;
         if ( status == PGRES_COMMAND_OK )
-                return 0;
+                return ntuple;
 
         return handle_error(PRELUDEDB_ERROR_QUERY, session);
 }
 
 
 
-static int sql_query(void *session, const char *query, preludedb_sql_table_t **table)
+static int sql_query_prepare(preludedb_sql_t *sql, preludedb_sql_query_t *query, prelude_string_t *output)
 {
         int ret;
-        PGresult *result;
+        int64_t limit = -1, offset = -1;
+
+        ret = prelude_string_cat(output, preludedb_sql_query_get_string(query));
+        if ( ret < 0 )
+                return ret;
+
+        if ( preludedb_sql_query_get_option(query, PRELUDEDB_SQL_QUERY_OPTION_FOR_UPDATE, NULL) ) {
+                ret = prelude_string_cat(output, " FOR UPDATE");
+                if ( ret < 0 )
+                        return ret;
+        }
+
+        preludedb_sql_query_get_option(query, PRELUDEDB_SQL_QUERY_OPTION_LIMIT, &limit);
+        preludedb_sql_query_get_option(query, PRELUDEDB_SQL_QUERY_OPTION_OFFSET, &offset);
+
+        return preludedb_sql_build_limit_offset_string(sql, limit, offset, output);
+}
+
+
+
+static int sql_query(void *session, const char *query, preludedb_sql_table_t **table)
+{
+        int ret, ret2;
+        PGresult *result = NULL;
 
         ret = _sql_query(session, query, &result);
-        if ( ret <= 0 )
+        if ( ret <= 0 || ! result )
                 return ret;
 
         if ( ! table )
                 PQclear(result);
         else {
-                ret = preludedb_sql_table_new(table, result);
-                if ( ret < 0 ) {
+                ret2 = preludedb_sql_table_new(table, result);
+                if ( ret2 < 0 ) {
                         PQclear(result);
-                        return ret;
+                        return ret2;
                 }
         }
 
-        return 1;
+        return ret;
 }
 
 
@@ -258,8 +287,10 @@ static int sql_escape(void *session, const char *input, size_t input_size, char
 
 #ifdef HAVE_PQESCAPESTRINGCONN
         rsize = PQescapeStringConn(session, (*output) + 1, input, input_size, &error);
-        if ( error )
+        if ( error ) {
+                free(*output);
                 return handle_error(PRELUDEDB_ERROR_GENERIC, session);
+        }
 #else
         rsize = PQescapeString((*output) + 1, input, input_size);
 #endif
@@ -433,7 +464,9 @@ static const char *get_operator_string(idmef_criterion_operator_t operator)
                 const char *name;
         } tbl[] = {
                 { IDMEF_CRITERION_OPERATOR_EQUAL,             "="           },
+                { IDMEF_CRITERION_OPERATOR_EQUAL_NOCASE,      "="           },
                 { IDMEF_CRITERION_OPERATOR_NOT_EQUAL,         "!="          },
+                { IDMEF_CRITERION_OPERATOR_NOT_EQUAL_NOCASE,  "!="          },
 
                 { IDMEF_CRITERION_OPERATOR_GREATER,           ">"           },
                 { IDMEF_CRITERION_OPERATOR_GREATER_OR_EQUAL,  ">="          },
@@ -474,16 +507,13 @@ static int sql_build_constraint_string(void *session, prelude_string_t *out, con
                 value = "";
 
         op_str = get_operator_string(operator);
-        if ( op_str )
-                return prelude_string_sprintf(out, "%s %s %s", field, op_str, value);
+        if ( ! op_str )
+                return preludedb_error_verbose(PRELUDEDB_ERROR_GENERIC, "Could not map operator to string");
 
-        else if ( operator == IDMEF_CRITERION_OPERATOR_EQUAL_NOCASE )
-                return prelude_string_sprintf(out, "lower(%s) = lower(%s)", field, value);
+        if ( operator == IDMEF_CRITERION_OPERATOR_EQUAL_NOCASE || operator == IDMEF_CRITERION_OPERATOR_NOT_EQUAL_NOCASE )
+                return prelude_string_sprintf(out, "lower(%s) %s lower(%s)", field, op_str, value);
 
-        else if ( operator == IDMEF_CRITERION_OPERATOR_NOT_EQUAL_NOCASE )
-                return prelude_string_sprintf(out, "lower(%s) != lower(%s)", field, value);
-
-        return -1;
+        return prelude_string_sprintf(out, "%s %s %s", field, op_str, value);
 }
 
 
@@ -564,7 +594,7 @@ static int sql_build_time_constraint_string(void *session, prelude_string_t *out
 
         sql_operator = get_operator_string(operator);
         if ( ! sql_operator )
-                return preludedb_error(PRELUDEDB_ERROR_GENERIC);
+                return preludedb_error_verbose(PRELUDEDB_ERROR_GENERIC, "Could not map operator to string");
 
         if ( type == PRELUDEDB_SQL_TIME_CONSTRAINT_WDAY )
                 value = value % 7 + 1;
@@ -644,6 +674,7 @@ int pgsql_LTX_preludedb_plugin_init(prelude_plugin_entry_t *pe, void *data)
         preludedb_plugin_sql_set_escape_func(plugin, sql_escape);
         preludedb_plugin_sql_set_escape_binary_func(plugin, sql_escape_binary);
         preludedb_plugin_sql_set_unescape_binary_func(plugin, sql_unescape_binary);
+        preludedb_plugin_sql_set_query_prepare_func(plugin, sql_query_prepare);
         preludedb_plugin_sql_set_query_func(plugin, sql_query);
         preludedb_plugin_sql_set_get_server_version_func(plugin, sql_get_server_version);
         preludedb_plugin_sql_set_table_destroy_func(plugin, sql_table_destroy);
