@@ -1,6 +1,6 @@
 /*****
 *
-* Copyright (C) 2005-2017 CS-SI. All Rights Reserved.
+* Copyright (C) 2005-2019 CS-SI. All Rights Reserved.
 * Author: Yoann Vandoorselaere <yoann.v@prelude-ids.com>
 *
 * This file is part of the PreludeDB library.
@@ -37,11 +37,11 @@ typedef struct preludedb_plugin_format preludedb_plugin_format_t;
 typedef int (*preludedb_plugin_format_check_schema_version_func_t)(const char *version);
 
 typedef int (*preludedb_plugin_format_get_alert_idents_func_t)(preludedb_t *db, idmef_criteria_t *criteria,
-                                                               int limit, int offset, preludedb_result_idents_order_t order,
+                                                               int limit, int offset, const preludedb_path_selection_t *order,
                                                                void **res);
 
 typedef int (*preludedb_plugin_format_get_heartbeat_idents_func_t)(preludedb_t *db, idmef_criteria_t *criteria,
-                                                                   int limit, int offset, preludedb_result_idents_order_t order,
+                                                                   int limit, int offset, const preludedb_path_selection_t *order,
                                                                    void **res);
 
 typedef size_t (*preludedb_plugin_format_get_message_ident_count_func_t)(void *res);
@@ -86,6 +86,8 @@ typedef int (*preludedb_plugin_format_path_resolve_func_t)(preludedb_selected_pa
 
 typedef int (*preludedb_plugin_format_init_func_t)(preludedb_t *db);
 
+typedef void (*preludedb_plugin_format_destroy_func_t)(preludedb_t *db);
+
 typedef int (*preludedb_plugin_format_optimize_func_t)(preludedb_t *db);
 
 
@@ -177,6 +179,8 @@ void preludedb_plugin_format_set_destroy_values_resource_func(preludedb_plugin_f
 
 void preludedb_plugin_format_set_init_func(preludedb_plugin_format_t *plugin, preludedb_plugin_format_init_func_t func);
 
+void preludedb_plugin_format_set_destroy_func(preludedb_plugin_format_t *plugin, preludedb_plugin_format_destroy_func_t func);
+
 void preludedb_plugin_format_set_optimize_func(preludedb_plugin_format_t *plugin, preludedb_plugin_format_optimize_func_t func);
 
 int preludedb_plugin_format_new(preludedb_plugin_format_t **ret);
