@@ -1,6 +1,6 @@
 /*****
 *
-* Copyright (C) 2003-2017 CS-SI. All Rights Reserved.
+* Copyright (C) 2003-2019 CS-SI. All Rights Reserved.
 * Author: Nicolas Delon <nicolas.delon@prelude-ids.com>
 *
 * This file is part of the PreludeDB library.
@@ -75,7 +75,15 @@ typedef enum {
 } preludedb_selected_object_interval_t;
 
 
+typedef enum {
+        PRELUDEDB_SQL_QUERY_OPTION_LIMIT      = 1,
+        PRELUDEDB_SQL_QUERY_OPTION_OFFSET     = 2,
+        PRELUDEDB_SQL_QUERY_OPTION_FOR_UPDATE = 3,
+} preludedb_sql_query_option_t;
+
+
 typedef struct preludedb_sql preludedb_sql_t;
+typedef struct preludedb_sql_query preludedb_sql_query_t;
 
 typedef struct preludedb_sql_table preludedb_sql_table_t;
 typedef struct preludedb_sql_row preludedb_sql_row_t;
@@ -113,6 +121,15 @@ int preludedb_sql_enable_query_logging(preludedb_sql_t *sql, const char *filenam
 void preludedb_sql_disable_query_logging(preludedb_sql_t *sql);
 
 int preludedb_sql_query(preludedb_sql_t *sql, const char *query, preludedb_sql_table_t **table);
+int preludedb_sql_query_new(preludedb_sql_query_t **query, const char *querystr);
+int preludedb_sql_query_set_option(preludedb_sql_query_t *query, int cmd, ...);
+int preludedb_sql_query_get_option(preludedb_sql_query_t *query, int cmd, void *data);
+
+preludedb_sql_query_t *preludedb_sql_query_ref(preludedb_sql_query_t *query);
+const char *preludedb_sql_query_get_string(preludedb_sql_query_t *query);
+int preludedb_sql_query_prepare(preludedb_sql_t *sql, preludedb_sql_query_t *query, prelude_string_t *output);
+int preludedb_sql_query_execute(preludedb_sql_t *sql, preludedb_sql_query_t *query, preludedb_sql_table_t **table);
+void preludedb_sql_query_destroy(preludedb_sql_query_t *query);
 
 int preludedb_sql_query_sprintf(preludedb_sql_t *sql, preludedb_sql_table_t **table, const char *format, ...)
                                 __attribute__ ((__format__ (__printf__, 3, 4)));
@@ -164,6 +181,8 @@ int preludedb_sql_field_to_float(preludedb_sql_field_t *field, float *value);
 int preludedb_sql_field_to_double(preludedb_sql_field_t *field, double *value);
 int preludedb_sql_field_to_string(preludedb_sql_field_t *field, prelude_string_t *output);
 
+const char *preludedb_sql_criteria_operator_to_string(idmef_criteria_operator_t op);
+
 int preludedb_sql_build_time_interval_string(preludedb_sql_t *sql, prelude_string_t *output, const char *field,
                                              const char *value, preludedb_selected_object_interval_t unit);
 
@@ -191,6 +210,16 @@ const char *preludedb_sql_get_type(const preludedb_sql_t *sql);
 const preludedb_sql_settings_t *preludedb_sql_get_settings(const preludedb_sql_t *sql);
 
 
+int preludedb_sql_connect(preludedb_sql_t *sql);
+int preludedb_sql_close(preludedb_sql_t *sql);
+
+void preludedb_sql_set_data(preludedb_sql_t *sql, void *data);
+
+void *preludedb_sql_get_data(preludedb_sql_t *sql);
+
+void *_preludedb_sql_get_plugin(preludedb_sql_t *sql);
+void *_preludedb_sql_get_session(preludedb_sql_t *sql);
+
 /*
  * Deprecated, use preludedb_strerror()
  */
