@@ -1,6 +1,6 @@
 /*****
 *
-* Copyright (C) 2003-2017 CS-SI. All Rights Reserved.
+* Copyright (C) 2003-2019 CS-SI. All Rights Reserved.
 * Author: Nicolas Delon <nicolas.delon@prelude-ids.com>
 *
 * This file is part of the PreludeDB library.
@@ -83,6 +83,7 @@ void preludedb_destroy(preludedb_t *db);
 
 const char *preludedb_get_format_name(preludedb_t *db);
 const char *preludedb_get_format_version(preludedb_t *db);
+const char *preludedb_get_format_uuid(preludedb_t *db);
 
 int preludedb_set_format(preludedb_t *db, const char *format_name);
 
@@ -99,13 +100,25 @@ char *preludedb_get_error(preludedb_t *db, preludedb_error_t error, char *errbuf
 
 int preludedb_insert_message(preludedb_t *db, idmef_message_t *message);
 
+void preludedb_set_data(preludedb_t *db, void *data);
+
+void *preludedb_get_data(preludedb_t *db);
+
 int preludedb_get_alert_idents(preludedb_t *db, idmef_criteria_t *criteria,
                                int limit, int offset,
                                preludedb_result_idents_order_t order,
+                               preludedb_result_idents_t **result) PRELUDE_DEPRECATED;
+int preludedb_get_alert_idents2(preludedb_t *db, idmef_criteria_t *criteria,
+                               int limit, int offset,
+                               const preludedb_path_selection_t *order,
                                preludedb_result_idents_t **result);
 int preludedb_get_heartbeat_idents(preludedb_t *db, idmef_criteria_t *criteria,
                                    int limit, int offset,
                                    preludedb_result_idents_order_t order,
+                                   preludedb_result_idents_t **result) PRELUDE_DEPRECATED;
+int preludedb_get_heartbeat_idents2(preludedb_t *db, idmef_criteria_t *criteria,
+                                   int limit, int offset,
+                                   const preludedb_path_selection_t *order,
                                    preludedb_result_idents_t **result);
 
 int preludedb_get_alert(preludedb_t *db, uint64_t ident, idmef_message_t **message);
