@@ -1,6 +1,6 @@
 /*****
 *
-* Copyright (C) 2003-2017 CS-SI. All Rights Reserved.
+* Copyright (C) 2003-2019 CS-SI. All Rights Reserved.
 * Author: Nicolas Delon <nicolas.delon@prelude-ids.com>
 *
 * This file is part of the PreludeDB library.
@@ -345,6 +345,24 @@ idmef_value_type_id_t preludedb_selected_object_get_value_type(preludedb_selecte
                         *dtype = object->type;
                         return IDMEF_VALUE_TYPE_UINT32;
 
+                case PRELUDEDB_SELECTED_OBJECT_TYPE_SUM:
+                        switch(preludedb_selected_object_get_value_type(object->data.args[0], data, dtype)) {
+                                case IDMEF_VALUE_TYPE_INT8:
+                                case IDMEF_VALUE_TYPE_INT16:
+                                case IDMEF_VALUE_TYPE_INT32:
+                                case IDMEF_VALUE_TYPE_INT64:
+                                        return IDMEF_VALUE_TYPE_INT64;
+
+                                case IDMEF_VALUE_TYPE_UINT8:
+                                case IDMEF_VALUE_TYPE_UINT16:
+                                case IDMEF_VALUE_TYPE_UINT32:
+                                case IDMEF_VALUE_TYPE_UINT64:
+                                        return IDMEF_VALUE_TYPE_UINT64;
+
+                                default:
+                                        return IDMEF_VALUE_TYPE_DOUBLE;
+                        }
+
                 case PRELUDEDB_SELECTED_OBJECT_TYPE_STRING:
                         *dtype = object->type;
                         *data = object->data.sval;
@@ -432,7 +450,7 @@ int preludedb_selected_path_get_column_index(preludedb_selected_path_t *selected
 }
 
 
-int preludedb_path_selection_get_selected(preludedb_path_selection_t *selection, preludedb_selected_path_t **selected, unsigned int index)
+int preludedb_path_selection_get_selected(const preludedb_path_selection_t *selection, preludedb_selected_path_t **selected, unsigned int index)
 {
         if ( index >= selection->count )
                 return preludedb_error_verbose(PRELUDEDB_ERROR_INDEX, "Invalid index '%u' for path selection", index);
@@ -512,7 +530,7 @@ int preludedb_path_selection_add(preludedb_path_selection_t *path_selection,
 
 
 
-preludedb_selected_path_t *preludedb_path_selection_get_next(preludedb_path_selection_t *path_selection,
+preludedb_selected_path_t *preludedb_path_selection_get_next(const preludedb_path_selection_t *path_selection,
                                                              preludedb_selected_path_t *selected_path)
 {
         int ret;
