@@ -1,6 +1,6 @@
 /*****
 *
-* Copyright (C) 2003-2017 CS-SI. All Rights Reserved.
+* Copyright (C) 2003-2019 CS-SI. All Rights Reserved.
 * Author: Nicolas Delon <nicolas.delon@prelude-ids.com>
 *
 * This file is part of the PreludeDB library.
@@ -98,8 +98,10 @@ struct preludedb_sql {
         prelude_bool_t internal_transaction_disabled;
         gl_recursive_lock_t mutex;
         int refcount;
+        void *data;
 };
 
+
 struct preludedb_sql_table {
         preludedb_sql_t *sql;
         void *data;
@@ -109,7 +111,7 @@ struct preludedb_sql_table {
         unsigned int row_count;
         unsigned int column_count;
 
-        uint16_t refcount;
+        uint32_t refcount;
         uint8_t done;
 };
 
@@ -130,6 +132,16 @@ struct preludedb_sql_row {
 };
 
 
+struct preludedb_sql_query {
+        int refcount;
+        char *string;
+
+        int64_t limit;
+        int64_t offset;
+        int for_update;
+};
+
+
 
 int _preludedb_sql_transaction_start(preludedb_sql_t *sql);
 int _preludedb_sql_transaction_end(preludedb_sql_t *sql);
@@ -156,6 +168,20 @@ static inline void update_sql_from_errno(preludedb_sql_t *sql, preludedb_error_t
 }
 
 
+void preludedb_sql_set_data(preludedb_sql_t *sql, void *data)
+{
+        prelude_return_if_fail(sql);
+        sql->data = data;
+}
+
+
+
+void *preludedb_sql_get_data(preludedb_sql_t *sql)
+{
+        prelude_return_val_if_fail(sql, NULL);
+        return sql->data;
+}
+
 
 /**
  * preludedb_sql_new:
@@ -295,10 +321,28 @@ void preludedb_sql_disable_query_logging(preludedb_sql_t *sql)
 
 
 
-static int preludedb_sql_connect(preludedb_sql_t *sql)
+
+int preludedb_sql_close(preludedb_sql_t *sql)
+{
+        if ( sql->status & PRELUDEDB_SQL_STATUS_CONNECTED ) {
+                _preludedb_plugin_sql_close(sql->plugin, sql->session);
+                sql->status &= ~PRELUDEDB_SQL_STATUS_CONNECTED;
+        }
+
+        return 0;
+}
+
+
+
+int preludedb_sql_connect(preludedb_sql_t *sql)
 {
         int ret;
 
+        if ( sql->status & PRELUDEDB_SQL_STATUS_CONNECTED ) {
+                _preludedb_plugin_sql_close(sql->plugin, sql->session);
+                sql->status &= ~PRELUDEDB_SQL_STATUS_CONNECTED;
+        }
+
         ret = _preludedb_plugin_sql_open(sql->plugin, sql->settings, &sql->session);
         if ( ret < 0 )
                 return ret;
@@ -337,15 +381,216 @@ void *preludedb_sql_table_get_data(preludedb_sql_table_t *table)
 
 
 /**
- * preludedb_sql_query:
+ * preludedb_sql_query_set_option:
+ * @query: Pointer to a query object.
+ * @cmd: The code of the option to set.
+ * @...: Variable argument list containing the value.
+ *
+ * Set the value of the specified option for the query @query.
+ * This value needs to be cast to the appropriate type by the caller.
+ *
+ * Returns: 0 on success or a negative value if an error occurs.
+ */
+int preludedb_sql_query_set_option(preludedb_sql_query_t *query, int cmd, ...)
+{
+        va_list va;
+
+        va_start(va, cmd);
+
+        switch (cmd) {
+                case PRELUDEDB_SQL_QUERY_OPTION_LIMIT:
+                    query->limit = va_arg(va, int64_t);
+                    break;
+
+                case PRELUDEDB_SQL_QUERY_OPTION_OFFSET:
+                    query->offset = va_arg(va, int64_t);
+                    break;
+
+                case PRELUDEDB_SQL_QUERY_OPTION_FOR_UPDATE:
+                    query->for_update = TRUE;
+                    break;
+
+                default:
+                    return prelude_error_verbose(PRELUDE_ERROR_GENERIC, "could not set query option: unknown value '%d'", cmd);
+        }
+
+        va_end(va);
+
+        return 0;
+}
+
+
+
+/**
+ * preludedb_sql_query_get_option:
+ * @query: Pointer to a query object.
+ * @cmd: The code of the option to retrieve.
+ * @ret: Pointer to an address where to store the option value.
+ *
+ * Get the value of the specified option for the query @query.
+ * This value will need to be cast to the appropriate type by the caller.
+ *
+ * Returns: 0 on success or a negative value if an error occurs.
+ */
+int preludedb_sql_query_get_option(preludedb_sql_query_t *query, int cmd, void *ret)
+{
+        switch (cmd) {
+                case PRELUDEDB_SQL_QUERY_OPTION_LIMIT:
+                    if ( ret )
+                            (*(int64_t *) ret) = query->limit;
+
+                    return (query->limit != -1) ? 1 : 0;
+
+                case PRELUDEDB_SQL_QUERY_OPTION_OFFSET:
+                    if ( ret )
+                            (*(int64_t *) ret) = query->offset;
+                    return (query->offset != -1) ? 1 : 0;
+
+                case PRELUDEDB_SQL_QUERY_OPTION_FOR_UPDATE:
+                    if ( ret )
+                            (*(int *) ret) = query->for_update;
+
+                    return (query->for_update) ? 1 : 0;
+
+                default:
+                    return prelude_error_verbose(PRELUDE_ERROR_GENERIC, "could not get query option: unknown value '%d'", cmd);
+        }
+
+        return 0;
+}
+
+
+
+/**
+ * preludedb_sql_query_new:
+ * @new: Pointer to a query object to initialize.
+ * @querystr: The SQL query.
+ *
+ * This function initialize the @new object.
+ *
+ * Returns: 0 on success or a negative value if an error occurs.
+ */
+int preludedb_sql_query_new(preludedb_sql_query_t **new, const char *querystr)
+{
+        *new = calloc(sizeof(**new), 1);
+        if ( ! *new )
+                return preludedb_error_from_errno(errno);
+
+        (*new)->string = strdup(querystr);
+        if ( ! (*new)->string ) {
+                free(*new);
+                return preludedb_error_from_errno(errno);
+        }
+
+        (*new)->limit = (*new)->offset = -1;
+        (*new)->refcount = 1;
+
+        return 0;
+}
+
+
+
+preludedb_sql_query_t *preludedb_sql_query_ref(preludedb_sql_query_t *query)
+{
+        query->refcount++;
+        return query;
+}
+
+
+
+/**
+ * preludedb_sql_query_get_string:
  * @sql: Pointer to a sql object.
- * @query: The SQL query to execute.
- * @table: Pointer to a table where the query result will be stored if the type of query return
- * results (i.e a SELECT can results, but an INSERT never results) and if the query is sucessfull.
+ * @query: Pointer to a query object.
+ *
+ * Get the original query string for the query @query.
+ *
+ * Returns: a pointer to the query string.
+ */
+const char *preludedb_sql_query_get_string(preludedb_sql_query_t *query)
+{
+        return query->string;
+}
+
+
+
+/**
+ * preludedb_sql_query_prepare:
+ * @sql: Pointer to a sql object.
+ * @query: Pointer to a query object.
+ * @output: Where the final query string will be stored.
+ *
+ * Get the final query string for the query @query.
+ *
+ * Returns: 0 on success or a negative value if an error occurs.
+ */
+int preludedb_sql_query_prepare(preludedb_sql_t *sql, preludedb_sql_query_t *query, prelude_string_t *output)
+{
+        return _preludedb_plugin_sql_query_prepare(sql, query, output);
+}
+
+
+
+/**
+ * preludedb_sql_query_execute:
+ * @sql: Pointer to a sql object.
+ * @query: Pointer to a query object.
+ * @table: Pointer to a table where the query result will be stored if the type of query returns
+ * results (i.e a SELECT can result, but an INSERT never results) and if the query is successful.
  *
  * Execute a SQL query.
  *
- * Returns: 1 if result are available, 0 for no result, -1 if an error occured.
+ * Returns: number of affected rows, -1 if an error occured.
+ */
+int preludedb_sql_query_execute(preludedb_sql_t *sql, preludedb_sql_query_t *query, preludedb_sql_table_t **table)
+{
+        int ret;
+        prelude_string_t *output;
+
+        ret = prelude_string_new(&output);
+        if ( ret < 0 )
+                return ret;
+
+        ret = preludedb_sql_query_prepare(sql, query, output);
+        if ( ret < 0 ) {
+                prelude_string_destroy(output);
+                return ret;
+        }
+
+        ret = preludedb_sql_query(sql, prelude_string_get_string(output), table);
+        prelude_string_destroy(output);
+        return ret;
+}
+
+
+
+/**
+ * preludedb_sql_query_destroy:
+ * @table: Pointer to a query object.
+ *
+ * Destroy the @query object.
+ */
+void preludedb_sql_query_destroy(preludedb_sql_query_t *query)
+{
+        if ( --query->refcount > 0 )
+                return;
+
+        free(query->string);
+        free(query);
+}
+
+
+
+/**
+ * preludedb_sql_query:
+ * @sql: Pointer to a sql object.
+ * @query: The SQL query to execute.
+ * @table: Pointer to a table where the query result will be stored if the type of query returns
+ * results (i.e a SELECT can results, but an INSERT never results) and if the query is successful.
+ *
+ * Execute a SQL query.
+ *
+ * Returns: number of affected rows, -1 if an error occurred.
  */
 int preludedb_sql_query(preludedb_sql_t *sql, const char *query, preludedb_sql_table_t **table)
 {
@@ -384,8 +629,10 @@ int preludedb_sql_query(preludedb_sql_t *sql, const char *query, preludedb_sql_t
         if ( ret <= 0 )
                 return ret;
 
-        (*table)->sql = preludedb_sql_ref(sql);
-        return 1;
+        if ( table && *table )
+                (*table)->sql = preludedb_sql_ref(sql);
+
+        return ret;
 }
 
 
@@ -1306,6 +1553,26 @@ int preludedb_sql_field_to_string(preludedb_sql_field_t *field, prelude_string_t
 }
 
 
+/**
+ * preludedb_sql_criteria_operator_to_string:
+ * @op: Boolean operator
+ *
+ * Get the SQL string representation of @op.
+ *
+ * Returns: the operator string or NULL if the operator @op could not be found.
+ */
+const char *preludedb_sql_criteria_operator_to_string(idmef_criteria_operator_t op)
+{
+        if ( op == IDMEF_CRITERIA_OPERATOR_AND )
+                return "AND";
+
+        else if ( op == IDMEF_CRITERIA_OPERATOR_OR )
+                return "OR";
+
+        return NULL;
+}
+
+
 static int build_criterion_fixed_sql_time_value(preludedb_sql_t *sql,
                                                 const idmef_value_t *value, char *buf, size_t size)
 {
@@ -1620,34 +1887,27 @@ int preludedb_sql_build_criterion_string(preludedb_sql_t *sql,
         int ret = -1;
         idmef_criterion_value_type_t type;
 
-        if ( operator == IDMEF_CRITERION_OPERATOR_NULL )
+        if ( operator == IDMEF_CRITERION_OPERATOR_NULL || operator == IDMEF_CRITERION_OPERATOR_NOT_NULL )
                 return prelude_string_sprintf(output, "%s %s", field,
                                               _preludedb_plugin_sql_get_operator_string(sql->plugin, operator));
 
-        else if ( operator == IDMEF_CRITERION_OPERATOR_NOT_NULL )
-                return prelude_string_sprintf(output, "%s %s", field,
-                                              _preludedb_plugin_sql_get_operator_string(sql->plugin, operator));
-
-        else if ( operator & IDMEF_CRITERION_OPERATOR_NOT ) {
-                ret = prelude_string_sprintf(output, "(%s %s OR ", field,
-                             _preludedb_plugin_sql_get_operator_string(sql->plugin, IDMEF_CRITERION_OPERATOR_NULL));
-                if ( ret < 0 )
-                        return ret;
-        }
-
         type = idmef_criterion_value_get_type(value);
+        switch (type) {
+                case IDMEF_CRITERION_VALUE_TYPE_VALUE:
+                        ret = build_criterion_fixed_value(sql, output, field, operator, idmef_criterion_value_get_value(value));
+                        break;
 
-        if ( type == IDMEF_CRITERION_VALUE_TYPE_VALUE )
-                ret = build_criterion_fixed_value(sql, output, field, operator, idmef_criterion_value_get_value(value));
+                case IDMEF_CRITERION_VALUE_TYPE_REGEX:
+                        ret = build_criterion_regex(sql, output, field, operator, idmef_criterion_value_get_regex(value));
+                        break;
 
-        else if ( type == IDMEF_CRITERION_VALUE_TYPE_REGEX )
-                ret = build_criterion_regex(sql, output, field, operator, idmef_criterion_value_get_regex(value));
+                case IDMEF_CRITERION_VALUE_TYPE_BROKEN_DOWN_TIME:
+                        ret = build_criterion_broken_down_time(sql, output, field, operator, idmef_criterion_value_get_broken_down_time(value));
+                        break;
 
-        else if ( type == IDMEF_CRITERION_VALUE_TYPE_BROKEN_DOWN_TIME )
-                ret = build_criterion_broken_down_time(sql, output, field, operator, idmef_criterion_value_get_broken_down_time(value));
-
-        if ( ret >= 0 && operator & IDMEF_CRITERION_OPERATOR_NOT )
-                ret = prelude_string_sprintf(output, ")");
+                default:
+                        ret = -1;
+        }
 
         return ret;
 }
@@ -1830,3 +2090,16 @@ void _preludedb_sql_disable_internal_transaction(preludedb_sql_t *sql)
 {
         sql->internal_transaction_disabled = TRUE;
 }
+
+
+
+void *_preludedb_sql_get_plugin(preludedb_sql_t *sql)
+{
+        return sql->plugin;
+}
+
+
+void *_preludedb_sql_get_session(preludedb_sql_t *sql)
+{
+        return sql->session;
+}
