@@ -1,6 +1,6 @@
 /*****
 *
-* Copyright (C) 2003-2017 CS-SI. All Rights Reserved.
+* Copyright (C) 2003-2019 CS-SI. All Rights Reserved.
 * Author: Nicolas Delon <nicolas.delon@prelude-ids.com>
 *
 * This file is part of the PreludeDB library.
@@ -28,6 +28,7 @@
 #include <stdlib.h>
 #include <sys/types.h>
 #include <libprelude/prelude.h>
+#include <libprelude/prelude-ident.h>
 #include <libprelude/idmef-criteria.h>
 
 
@@ -48,8 +49,10 @@
 struct preludedb {
         int refcount;
         char *format_version;
+        char *format_uuid;
         preludedb_sql_t *sql;
         preludedb_plugin_format_t *plugin;
+        void *data;
 };
 
 struct preludedb_result_idents {
@@ -141,17 +144,61 @@ void preludedb_deinit(void)
 
 
 
-static int preludedb_autodetect_format(preludedb_t *db)
+static int generate_uuid(preludedb_sql_t *sql, char **out)
 {
+        int ret;
+        prelude_ident_t *ident;
+        prelude_string_t *uuid;
+
+        ret = prelude_ident_new(&ident);
+        if ( ret < 0 )
+                return ret;
+
+        ret = prelude_string_new(&uuid);
+        if ( ret < 0 ) {
+                prelude_ident_destroy(ident);
+                return ret;
+        }
+
+        ret = prelude_ident_generate(ident, uuid);
+        if ( ret < 0 )
+                goto error;
+
+        ret = prelude_string_get_string_released(uuid, out);
+        if ( ret < 0 )
+                goto error;
+
+        ret = preludedb_sql_query_sprintf(sql, NULL, "UPDATE _format SET uuid = '%s'", *out);
+
+ error:
+        prelude_ident_destroy(ident);
+        prelude_string_destroy(uuid);
+
+        return ret;
+}
+
+
+static int _preludedb_autodetect_format(preludedb_t *db)
+{
+        int ret;
+        preludedb_sql_query_t *query;
         preludedb_sql_table_t *table;
         preludedb_sql_row_t *row;
         preludedb_sql_field_t *format_name;
         preludedb_sql_field_t *format_version;
-        int ret;
+        preludedb_sql_field_t *format_uuid;
 
-        ret = preludedb_sql_query(db->sql, "SELECT name, version from _format", &table);
-        if ( ret <= 0 )
+        ret = preludedb_sql_query_new(&query, "SELECT name, version, uuid FROM _format");
+        if ( ret < 0 )
+                return ret;
+
+        preludedb_sql_query_set_option(query, PRELUDEDB_SQL_QUERY_OPTION_FOR_UPDATE);
+
+        ret = preludedb_sql_query_execute(db->sql, query, &table);
+        if ( ret <= 0 ) {
+                preludedb_sql_query_destroy(query);
                 return (ret < 0) ? ret : -1;
+        }
 
         ret = preludedb_sql_table_fetch_row(table, &row);
         if ( ret < 0 )
@@ -174,10 +221,23 @@ static int preludedb_autodetect_format(preludedb_t *db)
                 goto error;
 
         db->format_version = strdup(preludedb_sql_field_get_value(format_version));
-        if ( ! db->format_version )
+        if ( ! db->format_version ) {
                 ret = prelude_error_from_errno(errno);
+                goto error;
+        }
+
+        ret = preludedb_sql_row_get_field(row, 2, &format_uuid);
+        if ( ret < 0 )
+                goto error;
+
+        if ( format_uuid )
+                db->format_uuid = strdup(preludedb_sql_field_get_value(format_uuid));
+        else {
+                ret = generate_uuid(db->sql, &db->format_uuid);
+        }
 
  error:
+        preludedb_sql_query_destroy(query);
         preludedb_sql_table_destroy(table);
 
         return ret;
@@ -185,6 +245,41 @@ static int preludedb_autodetect_format(preludedb_t *db)
 }
 
 
+static int preludedb_autodetect_format(preludedb_t *db)
+{
+        int ret, tmp;
+
+        ret = preludedb_transaction_start(db);
+        if ( ret < 0 ) {
+                return ret;
+        }
+
+        ret = _preludedb_autodetect_format(db);
+        if ( ret < 0 ) {
+                tmp = preludedb_transaction_abort(db);
+                return (tmp < 0) ? tmp : ret;
+        }
+
+        return preludedb_transaction_end(db);
+}
+
+
+
+void preludedb_set_data(preludedb_t *db, void *data)
+{
+        prelude_return_if_fail(db);
+        db->data = data;
+}
+
+
+
+void *preludedb_get_data(preludedb_t *db)
+{
+        prelude_return_val_if_fail(db, NULL);
+        return db->data;
+}
+
+
 
 /**
  * preludedb_new:
@@ -211,28 +306,26 @@ int preludedb_new(preludedb_t **db, preludedb_sql_t *sql, const char *format_nam
         *db = calloc(1, sizeof (**db));
         if ( ! *db ) {
                 ret = preludedb_error_from_errno(errno);
-                snprintf(errbuf, size, "%s", preludedb_strerror(ret));
                 return ret;
         }
 
         (*db)->refcount = 1;
         (*db)->sql = preludedb_sql_ref(sql);
 
-        if ( format_name )
-                ret = preludedb_set_format(*db, format_name);
-        else
-                ret = preludedb_autodetect_format(*db);
+        ret = preludedb_autodetect_format(*db);
 
         if ( ret >= 0 && (*db)->plugin->init )
                 ret = (*db)->plugin->init(*db);
 
         if ( ret < 0 ) {
-                if ( errbuf )
-                        preludedb_get_error(*db, ret, errbuf, size);
+                preludedb_sql_destroy(sql);
 
                 if ( (*db)->format_version )
                         free((*db)->format_version);
 
+                if ( (*db)->format_uuid )
+                        free((*db)->format_uuid);
+
                 free(*db);
         }
 
@@ -262,8 +355,12 @@ void preludedb_destroy(preludedb_t *db)
         if ( --db->refcount != 0 )
                 return;
 
+        if ( db->plugin && db->plugin->destroy_func )
+                db->plugin->destroy_func(db);
+
         preludedb_sql_destroy(db->sql);
         free(db->format_version);
+        free(db->format_uuid);
         free(db);
 }
 
@@ -297,6 +394,19 @@ const char *preludedb_get_format_version(preludedb_t *db)
 
 
 
+/**
+ * preludedb_get_format_uuid:
+ * @db: Pointer to a db object.
+ *
+ * Returns: the UUID for this database.
+ */
+const char *preludedb_get_format_uuid(preludedb_t *db)
+{
+        prelude_return_val_if_fail(db, NULL);
+        return db->format_uuid;
+}
+
+
 /**
  * preludedb_set_format:
  * @db: Pointer to a db object.
@@ -535,15 +645,47 @@ int preludedb_result_values_get_field_direct(preludedb_result_values_t *result,
 }
 
 
+static int get_message_idents_prepare_order(preludedb_t *db, idmef_class_id_t message_type,
+                                            preludedb_result_idents_order_t order,
+                                            preludedb_path_selection_t **path_selection)
+{
+        int ret;
+        preludedb_selected_path_t *selected_path;
+        const char *path = (message_type == IDMEF_CLASS_ID_ALERT) ? "alert.create_time": "heartbeat.create_time";
+
+        ret = preludedb_path_selection_new(db, path_selection);
+        if ( ret < 0 )
+                return ret;
+
+        ret = preludedb_selected_path_new_string(&selected_path, path);
+        if ( ret < 0 ) {
+                preludedb_path_selection_destroy(*path_selection);
+                return ret;
+        }
+
+        preludedb_selected_path_set_flags(selected_path,
+                                          (order == PRELUDEDB_RESULT_IDENTS_ORDER_BY_CREATE_TIME_DESC) ?
+                                           PRELUDEDB_SELECTED_PATH_FLAGS_ORDER_DESC : PRELUDEDB_SELECTED_PATH_FLAGS_ORDER_ASC);
+
+        ret = preludedb_path_selection_add(*path_selection, selected_path);
+        if ( ret < 0 ) {
+                preludedb_path_selection_destroy(*path_selection);
+                preludedb_selected_path_destroy(selected_path);
+        }
+
+        return ret;
+}
+
+
 static int
 preludedb_get_message_idents(preludedb_t *db,
                              idmef_criteria_t *criteria,
                              int (*get_idents)(preludedb_t *db, idmef_criteria_t *criteria,
                                                int limit, int offset,
-                                               preludedb_result_idents_order_t order,
+                                               const preludedb_path_selection_t *order,
                                                void **res),
                              int limit, int offset,
-                             preludedb_result_idents_order_t order,
+                             const preludedb_path_selection_t *order,
                              preludedb_result_idents_t **result)
 {
         int ret;
@@ -581,6 +723,39 @@ int preludedb_get_alert_idents(preludedb_t *db,
                                idmef_criteria_t *criteria, int limit, int offset,
                                preludedb_result_idents_order_t order,
                                preludedb_result_idents_t **result)
+{
+        int ret;
+        preludedb_path_selection_t *order_selection;
+
+        prelude_return_val_if_fail(db && result, prelude_error(PRELUDE_ERROR_ASSERTION));
+
+        ret = get_message_idents_prepare_order(db, IDMEF_CLASS_ID_ALERT, order, &order_selection);
+        if ( ret < 0 )
+                return ret;
+
+        ret = preludedb_get_message_idents(db, criteria, db->plugin->get_alert_idents, limit, offset, order_selection, result);
+        preludedb_path_selection_destroy(order_selection);
+
+        return ret;
+}
+
+
+
+/**
+ * preludedb_get_alert_idents2:
+ * @db: Pointer to a db object.
+ * @criteria: Pointer to an idmef criteria.
+ * @limit: Limit of results or -1 if no limit.
+ * @offset: Offset in results or -1 if no offset.
+ * @order: Result order.
+ * @result: Idents result.
+ *
+ * Returns: the number of result or a negative value if an error occured.
+ */
+int preludedb_get_alert_idents2(preludedb_t *db,
+                                idmef_criteria_t *criteria, int limit, int offset,
+                                const preludedb_path_selection_t *order,
+                                preludedb_result_idents_t **result)
 {
         prelude_return_val_if_fail(db && result, prelude_error(PRELUDE_ERROR_ASSERTION));
         return preludedb_get_message_idents(db, criteria, db->plugin->get_alert_idents, limit, offset, order, result);
@@ -603,6 +778,39 @@ int preludedb_get_heartbeat_idents(preludedb_t *db,
                                    idmef_criteria_t *criteria, int limit, int offset,
                                    preludedb_result_idents_order_t order,
                                    preludedb_result_idents_t **result)
+{
+        int ret;
+        preludedb_path_selection_t *order_selection;
+
+        prelude_return_val_if_fail(db && result, prelude_error(PRELUDE_ERROR_ASSERTION));
+
+        ret = get_message_idents_prepare_order(db, IDMEF_CLASS_ID_HEARTBEAT, order, &order_selection);
+        if ( ret < 0 )
+                return ret;
+
+        ret = preludedb_get_message_idents(db, criteria, db->plugin->get_heartbeat_idents, limit, offset, order_selection, result);
+        preludedb_path_selection_destroy(order_selection);
+
+        return ret;
+}
+
+
+
+/**
+ * preludedb_get_heartbeat_idents2:
+ * @db: Pointer to a db object.
+ * @criteria: Pointer to an idmef criteria.
+ * @limit: Limit of results or -1 if no limit.
+ * @offset: Offset in results or -1 if no offset.
+ * @order: Result order.
+ * @result: Idents result.
+ *
+ * Returns: the number of result or a negative value if an error occured.
+ */
+int preludedb_get_heartbeat_idents2(preludedb_t *db,
+                                    idmef_criteria_t *criteria, int limit, int offset,
+                                    const preludedb_path_selection_t *order,
+                                    preludedb_result_idents_t **result)
 {
         prelude_return_val_if_fail(db && result, prelude_error(PRELUDE_ERROR_ASSERTION));
         return preludedb_get_message_idents(db, criteria, db->plugin->get_heartbeat_idents, limit, offset, order, result);
@@ -774,7 +982,7 @@ int preludedb_delete(preludedb_t *db, idmef_criteria_t *criteria)
                 return ret;
 
         if ( ret == IDMEF_CLASS_ID_ALERT ) {
-                ret = preludedb_get_alert_idents(db, criteria, -1, 0, 0, &result);
+                ret = preludedb_get_alert_idents2(db, criteria, -1, 0, NULL, &result);
                 if ( ret <= 0 )
                         return ret;
 
@@ -782,7 +990,7 @@ int preludedb_delete(preludedb_t *db, idmef_criteria_t *criteria)
         }
 
         else if ( IDMEF_CLASS_ID_HEARTBEAT ) {
-                ret = preludedb_get_heartbeat_idents(db, criteria, -1, 0, 0, &result);
+                ret = preludedb_get_heartbeat_idents2(db, criteria, -1, 0, NULL, &result);
                 if ( ret <= 0 )
                         return ret;
 
